#include <X3DTK/kernel.h>
#include <X3DTK/X3D/scenegraph.h>
#include <X3DTK/X3D/meshbuilder.h>
#include <X3DTK/MESH/statscalculator.h>

#include <iostream>

using namespace X3DTK;
using namespace std;

int main(int argc, char *argv[])
{
  if (argc <= 1)
  {
    cerr << "usage: X3DStats input" << endl;
    exit(0);
  }

  // DefaultLoader to load the default X3D Nodes.
  X3D::Loader *loader = X3DLoader::getInstanceOf<X3D::Loader>();
  X3D::DefMeshBuilder *meshBuilder = X3DProcessor::getInstanceOf<X3D::DefMeshBuilder>();
  MESH::DefStatsCalculator *calculator = X3DProcessor::getInstanceOf<MESH::DefStatsCalculator>();
  
  // Loads the scene.
  X3D::Scene *s = loader->load(argv[1], true);
  MESH::Scene *ms = meshBuilder->build(s);
  calculator->calculate(ms);

  const MESH::DefMeshInfo &modelMeshInfo = calculator->getModelMeshInfo();
  const MESH::DefVertexInfo &modelVertexInfo = calculator->getModelVertexInfo();
  
  cout << argv[1] << " statistics:" << endl;    
  cout << "  faces : " << modelMeshInfo.getFaces() << endl;
  cout << "  vertices : " << modelVertexInfo.getVertices() << endl;
  cout << "  edges : " << modelMeshInfo.getEdges() << endl;
  
  const MESH::DefMeshInfoList &meshInfoList = calculator->getMeshInfoList();
  const MESH::DefVertexInfoList &vertexInfoList = calculator->getVertexInfoList();
  
  for (MESH::DefMeshInfoList::const_iterator it = meshInfoList.begin(); it != meshInfoList.end(); ++it)
  {
    if ((*it).second.getInstances() == 1)
      cout << "-> 1 instance" << endl;
    else
      cout << "-> " << (*it).second.getInstances() << " instances" << endl;      
    cout << "  faces : " << (*it).second.getFaces() << endl;
    cout << "  edges : " << (*it).second.getEdges() << endl << endl;
  }
  
  return 1;
}
